@@ -3,7 +3,9 @@ Routes for credit card financials.
 """
 from collections import Counter
 
-from flask import redirect, render_template, flash, request, url_for, jsonify
+from flask import (
+    g, redirect, render_template, flash, request, url_for, jsonify
+)
 from werkzeug.exceptions import abort
 
 from monopyly.db import get_db
@@ -26,7 +28,7 @@ form_err_msg = 'There was an improper value in your form. Please try again.'
 
 @credit.route('/cards')
 @login_required
-def show_cards():
+def load_cards():
     card_db = CardHandler()
     # Get all of the user's credit cards from the database
     cards = card_db.get_entries()
@@ -45,7 +47,7 @@ def new_card():
             card_db = CardHandler()
             # Insert the new credit card into the database
             card = card_db.add_entry(form.card_data)
-            return redirect(url_for('credit.show_account',
+            return redirect(url_for('credit.load_account',
                                     account_id=card['account_id']))
         else:
             flash(form_err_msg)
@@ -55,7 +57,7 @@ def new_card():
 
 @credit.route('/account/<int:account_id>')
 @login_required
-def show_account(account_id):
+def load_account(account_id):
     account_db, card_db = AccountHandler(), CardHandler()
     # Get the account information from the database
     account = account_db.get_entry(account_id)
@@ -90,7 +92,7 @@ def delete_card(card_id):
     account_id = card_db.get_entry(card_id)['account_id']
     # Remove the credit card from the database
     card_db.delete_entries((card_id,))
-    return redirect(url_for('credit.show_account', account_id=account_id))
+    return redirect(url_for('credit.load_account', account_id=account_id))
 
 
 @credit.route('/_update_account_statement_issue_day/<int:account_id>',
@@ -125,12 +127,12 @@ def delete_account(account_id):
     account_db = AccountHandler()
     # Remove the account from the database
     account_db.delete_entries((account_id,))
-    return redirect(url_for('credit.show_cards'))
+    return redirect(url_for('credit.load_cards'))
 
 
 @credit.route('/statements')
 @login_required
-def show_statements():
+def load_statements():
     card_db, statement_db = CardHandler(), StatementHandler()
     # Get all of the user's credit cards from the database
     all_cards = card_db.get_entries()
@@ -164,8 +166,10 @@ def update_statements_display():
 
 @credit.route('/statement/<int:statement_id>')
 @login_required
-def show_statement(statement_id):
-    statement_db, transaction_db = StatementHandler(), TransactionHandler()
+def load_statement(statement_id):
+    statement_db = StatementHandler()
+    transaction_db = TransactionHandler()
+    tag_db = TagHandler()
     # Get the statement information from the database
     statement_fields = ('account_id', 'card_id', 'bank', 'last_four_digits',
                         'issue_date', 'due_date', 'balance', 'payment_date')
@@ -176,9 +180,15 @@ def show_statement(statement_id):
     transactions = transaction_db.get_entries(statement_ids=(statement['id'],),
                                               sort_order=sort_order,
                                               fields=transaction_fields)
+    tag_totals = tag_db.get_totals(statement_ids=(statement['id'],))
+    tag_totals = {row['tag_name']: row['total'] for row in tag_totals}
+    tag_avgs = tag_db.get_statement_average_totals()
+    tag_avgs = {row['tag_name']: row['average_total'] for row in tag_avgs}
     return render_template('credit/statement_page.html',
                            statement=statement,
-                           statement_transactions=transactions)
+                           statement_transactions=transactions,
+                           tag_totals=tag_totals,
+                           tag_average_totals=tag_avgs)
 
 
 @credit.route('/_update_statement_due_date/<int:statement_id>',
@@ -208,7 +218,7 @@ def make_payment(card_id, statement_id):
     # Add the paymnet as a transaction in the database
     card = card_db.get_entry(card_id)
     statement = statement_db.infer_statement(card, payment_date, creation=True)
-    transaction_db.add_transaction(statement=statement,
+    transaction_db.add_transaction(statement_id=statement['id'],
                                    transaction_date=payment_date,
                                    vendor=card['bank'],
                                    amount=-payment_amount,
@@ -223,7 +233,7 @@ def make_payment(card_id, statement_id):
 
 @credit.route('/transactions')
 @login_required
-def show_transactions():
+def load_transactions():
     card_db, transaction_db = CardHandler(), TransactionHandler()
     # Get all of the user's credit cards from the database
     cards = card_db.get_entries()
@@ -295,7 +305,7 @@ def new_transaction(statement_id):
             transaction_db, tag_db = TransactionHandler(), TagHandler()
             # Insert the new transaction into the database
             transaction = transaction_db.add_entry(form.transaction_data)
-            tag_db.update_tags(transaction, form.tag_data)
+            tag_db.update_tags(transaction['id'], form.tag_data)
             return render_template('credit/transaction_submission_page.html',
                                    transaction=transaction, update=False)
         else:
@@ -325,7 +335,7 @@ def update_transaction(transaction_id):
             # Update the database with the updated transaction
             transaction = transaction_db.update_entry(transaction_id,
                                                       form.transaction_data)
-            tag_db.update_tags(transaction, form.tag_data)
+            tag_db.update_tags(transaction['id'], form.tag_data)
             return render_template('credit/transaction_submission_page.html',
                                    transaction=transaction, update=True)
         else:
@@ -343,7 +353,50 @@ def delete_transaction(transaction_id):
     transaction_db = TransactionHandler()
     # Remove the transaction from the database
     transaction_db.delete_entries((transaction_id,))
-    return redirect(url_for('credit.show_transactions'))
+    return redirect(url_for('credit.load_transactions'))
+
+
+@credit.route('/tags')
+@login_required
+def load_tags():
+    tag_db = TagHandler()
+    # Get the tag heirarchy from the database
+    heirarchy = tag_db.get_heirarchy()
+    return render_template('credit/tags_page.html',
+                           tags_heirarchy=heirarchy)
+
+@credit.route('/_new_tag', methods=('POST',))
+@login_required
+def new_tag():
+    tag_db = TagHandler()
+    # Get the new tag (and potentially parent category) from the AJAX request
+    post_args = request.get_json()
+    tag_name = post_args['tag_name']
+    parent_name = post_args.get('parent')
+    # Check that the tag name does not already exist
+    if tag_db.get_entries(tag_names=(tag_name,)):
+        raise ValueError('The given tag name already exists. Tag names must '
+                         'be unique.')
+    if parent_name:
+        parent_id = tag_db.find_tag(parent_name, fields=())['id']
+    else:
+        parent_id = None
+    tag_data = {'parent_id': parent_id,
+                'user_id': g.user['id'],
+                'tag_name': tag_name}
+    tag = tag_db.add_entry(tag_data)
+    return render_template('credit/subtag_tree.html',
+                           tag=(tag['id'], tag['tag_name']),
+                           tags_heirarchy={})
+
+
+@credit.route('/delete_tag/<int:tag_id>')
+@login_required
+def delete_tag(tag_id):
+    tag_db = TagHandler()
+    # Remove the tag from the database
+    tag_db.delete_entries((tag_id,))
+    return redirect(url_for('credit.load_tags'))
 
 
 @credit.route('/_suggest_autocomplete', methods=('POST',))
